﻿KLS = LibStub("AceAddon-3.0"):NewAddon("Keyboard Layout Saver", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")
KLS:SetDefaultModuleState(false)

AceGUI = LibStub("AceGUI-3.0")


-- Globals within this addon
local KLSversion = "1.7.2"
local KLSdebugframe = 0
local KLSdebugbuffer = {}
local KLSdebugEnabled = 0
local run_counter = {}      -- Global for routines to not run more than once, modified by event_throttle() ONLY!
local update_bindings_counter = 0 -- UPDATE_BINDINGS event must not run first time

function KLS:Echo(text)
   DEFAULT_CHAT_FRAME:AddMessage("KLS: "..text)
end


function KLS:OnInitialize()
   
    KLSdb = LibStub("AceDB-3.0"):New("KeyboardLayoutSaver",nil,"Default")
    KLS:SetupOptions()
	
        -- Check if we have are supposed to load the addon
    if KLSdb.char.KLSaddonEnabled == 0 then 
        KLS:Disable()
    else
        KLS:Enable()
    end
end

-- Addon loaded
function KLS:OnEnable()
KLS:DebugMsg("Core: OnEnable()")
    
    KLSdebugEnabled = KLSdb.char.KLSdebugEnabled or 0
    KLS:DebugWindowToggle(KLSdebugEnabled)
    
    DEFAULT_CHAT_FRAME:AddMessage("Keyboard Layout Saver v"..KLSversion.." ON")
    
    if KLSdb.char.KLSaddonEnabled == nil then
        KLSdb.char.KLSaddonEnabled = 1
    end
    
    KLSdb.char.KLStriggerModule = KLSdb.char.KLStriggerModule or "KLS: Talents"
    KLS:DebugMsg("Core: OnEnable() - Loading trigger module -"..KLSdb.char.KLStriggerModule.."-") 
    
    KLS:EnableModule(KLSdb.char.KLStriggerModule)
    KLS:RegisterEvent("UPDATE_BINDINGS")
    
    -----------------------------------------------------
	-- Unregister UPDATE_BINDINGS if we go into combat. -
	-----------------------------------------------------
    KLS:RegisterEvent("PLAYER_ENTER_COMBAT") -- Are we in combat? Disable UPDATE_BINDINGS!
    KLS:RegisterEvent("PLAYER_REGEN_DISABLED")
    
    KLS:RegisterEvent("PLAYER_LEAVE_COMBAT") -- Enable UPDATE_BINDINGS if we exit the fight
    KLS:RegisterEvent("PLAYER_REGEN_ENABLED")
    
   
end


-----------------------------------------------------
-- Unregister UPDATE_BINDINGS if we go into combat. -
-----------------------------------------------------
function KLS:PLAYER_ENTER_COMBAT()
--	KLS:DebugMsg("PLAYER_ENTER_COMBAT(): Entered")

    KLS:UnregisterEvent("UPDATE_BINDINGS")
end

function KLS:PLAYER_LEAVE_COMBAT()
--	KLS:DebugMsg("PLAYER_LEAVE_COMBAT(): Entered")

    -- Register again as we have now exited combat!
    update_bindings_counter = 0 -- Do not run the first time after we register this event!
    KLS:RegisterEvent("UPDATE_BINDINGS")
end

function KLS:PLAYER_REGEN_DISABLED()
--	KLS:DebugMsg("PLAYER_REGEN_DISABLED(): Entered")

    KLS:UnregisterEvent("UPDATE_BINDINGS")
end

function KLS:PLAYER_REGEN_ENABLED()
--	KLS:DebugMsg("PLAYER_REGEN_ENABLED(): Entered")

    -- Register again as we have now exited combat!
    update_bindings_counter = 0 -- Do not run the first time after we register this event!
    KLS:RegisterEvent("UPDATE_BINDINGS")
end

-- Toggle a named module.
-- If no module is passed to this routine, it shuts off all modules 
-- and sets KLS: Talents as default
function KLS:ModuleToggle(KLSmodule)
KLS:DebugMsg("ModuleToggle()")

    -- Empty named var, setting default
    if not KLSmodule then
        KLS:DebugMsg("ModuleToggle(): No module passed on. Setting default..")
        
        -- Disable all but default
        for name, module in KLS:IterateModules() do
        
        
            if name ~= "KLS: Talents" then
                KLS:DebugMsg("ModuleToggle(): Disabling -"..name.."-")
                KLS:DisableModule(name)
            end
        end
        
        KLSdb.char.KLStriggerModule = "KLS: Talents"
        KLS:EnableModule("KLS: Talents")
        
        return
    end
    
    -- This module is enabled, disable it now
    if KLSdb.char.KLStriggerModule == KLSmodule then
    KLS:DebugMsg("ModuleToggle(): This is already enabled. Now disabling -"..KLSmodule)
    
        -- Disable all but default
        for name, module in KLS:IterateModules() do
            if name ~= "KLS: Talents" then
                KLS:DebugMsg("ModuleToggle(): Disabling -"..name.."-")
                KLS:DisableModule(name)
            end
        end
        
        KLSdb.char.KLStriggerModule = "KLS: Talents"
        KLS:EnableModule("KLS: Talents")
        
        return
    end
    
    -- This module is disabled, try to enable it!
    
    -- Disable all but this module
    for name, module in KLS:IterateModules() do   
            if name ~= KLSmodule then
                KLS:DebugMsg("ModuleToggle(): Disabling -"..name.."-")
                KLS:DisableModule(name)
            else
                KLS:DebugMsg("ModuleToggle(): Enabling -"..KLSmodule.."-")
                
            end
    end
    
    KLS:DebugMsg("ModuleToggle(): This is disabled. Now enabling -"..KLSmodule)
    
    KLSdb.char.KLStriggerModule = KLSmodule
    
    if not KLS:EnableModule(KLSdb.char.KLStriggerModule) then
        KLS:DebugMsg("ModuleToggle(): Loading module '"..KLSdb.char.KLStriggerModule.."' failed. Reverting to 'KLS: Talents' module.")
        DEFAULT_CHAT_FRAME:AddMessage(KLSdb.char.KLStriggerModule.." failed to load. Setting default -KLS: Talents-")
        
        KLSdb.char.KLStriggerModule = "KLS: Talents"
        KLS:EnableModule("KLS: Talents")

        return
    else
        KLS:DebugMsg("ModuleToggle(): Module '"..KLSdb.char.KLStriggerModule.."' loaded.")
    end
end

function KLS:OnDisable()
KLS:DebugMsg("OnDisable()")
        
    DEFAULT_CHAT_FRAME:AddMessage("Keyboard Layout Saver v"..KLSversion.." OFF")
   
    KLS:DebugMsg("OnDisable(): Unregistering modules")
    KLS:DisableModule(KLSdb.char.KLStriggerModule)
    
    KLSdb.char.KLSaddonEnabled = 0
    KLS:Disable()
end





function KLS:ExistsProfile(name)
--KLS:DebugMsg("ExistsProfile("..name..")")
   for i, k in ipairs(KLSdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end

--User updated his keybinds
function KLS:UPDATE_BINDINGS(...) -- Should not fire first time!
	KLS:DebugMsg("UPDATE_BINDINGS()")
    
    -- UPDATE_BINDINGS event must not run first time
    if update_bindings_counter == 0 then
		KLS:DebugMsg("UPDATE_BINDINGS(): First time run, returning...")
		update_bindings_counter = 1
		return
    end
	
    KLS:DebugMsg("UPDATE_BINDINGS(): Saving keybinds for profile: -"..KLSdb:GetCurrentProfile().."-")
    KLS:SaveProfil(KLSdb:GetCurrentProfile())
end

function KLS:SaveProfil(name)

	if name == nil then
		name = KLSdb:GetCurrentProfile()
	end
	
	KLS:DebugMsg("SaveProfil("..name..")")

	KLSdb:SetProfile(name)
	KLSdb.profile["Reset"] = 0
        
    -- Go through the actionbars  and save _defined_ keybinds// Thank you Adirelle
	for index = 1, GetNumBindings() do
	  local command, key1 = GetBinding(index)
	  
	  if key1 then
		-- This creates a table like { cmd = key1, key2, key3 }
		KLSdb.profile[command] = { key1, select(3, GetBinding(index)) }
	  else
		-- Remove previous bindings
		KLSdb.profile[command] = nil
	  end
	end  

end

function KLS:LoadProfil(name)
KLS:DebugMsg("LoadProfil("..name..")")

KLS:UnregisterEvent("UPDATE_BINDINGS")

	if (UnitAffectingCombat("player")~=nil)then -- if (player is in fight)
		KLS:Echo("You can t change your keyboardlayout in fight.")
	elseif (KLS:ExistsProfile(name)==nil)then
		KLS:Echo("-"..name.."- doesn t exist.")
	elseif (KLSdb:GetCurrentProfile()==name)then
		KLS:DebugMsg("LoadProfil(): "..name.." already loaded.")
	else
		KLS:DebugMsg("LoadProfil(): Setting profile "..name)
        KLSdb:SetProfile(name)
        
        -- To restore
		for command, keys in pairs(KLSdb.profile) do
            
            
            if command ~= "Reset" then 		
					local current_keys = {GetBindingKey(command)}       -- Current bindings -> UP/W
					local saved_keys = {}                               -- Saved bindings
					local binding_modified = 0                          -- Modified flag
					local current_keys_table = {}

					-- Transform to hash tables
					for i,v in pairs(KLSdb.profile[command]) do saved_keys[v]="1" end			 -- table[1]:W -> saved_keys[W]
					for i=1, #current_keys do  current_keys_table[current_keys[i]] = "1"  end    -- @table     -> current_keys_table[Q]

					-- Check saved <=> current hash
					for i,v in pairs (saved_keys) do
					   if not current_keys_table[i] then
						  binding_modified = 1
						  KLS:DebugMsg("LoadProfil(): .Key "..i.." was changed in saved profile.")
					   end
					end

					-- Check current <=> saved hash
					if (binding_modified == 0) then
					   for i,v in pairs (current_keys_table) do
						  if not saved_keys[i] then
							 binding_modified = 1    
							  KLS:DebugMsg("LoadProfil(): -Key "..i.." was changed in saved profile.")
						  end
					   end
					end
					
					-- Modified flag active!
					if binding_modified == 1 then
					KLS:DebugMsg("LoadProfil(): "..command.." was found changed. Now restoring..")
						-- Remove all current binds for this action (eg. MOVEFORWARD)
						for i,v in pairs (current_keys_table) do
						   --KLS:DebugMsg("LoadProfil(): +Key "..i.." was changed in saved profile.")
						   SetBinding(i)
						end
						-- Add all saved keys to this action
						for i,v in pairs (saved_keys) do
							SetBinding(i,command)
							--KLS:DebugMsg("LoadProfil(): >Key "..i.." was changed in saved profile.")
						end
						binding_modified = 0
				   end
			 end
		end          
                  
	end



    KLS:Echo("-"..name.."- loaded")
    SaveBindings(2)
    
KLS:RegisterEvent("UPDATE_BINDINGS")
end

function KLS:DeleteProfil(name)
KLS:DebugMsg("DeleteProfil("..name..")")

   if (KLS:ExistsProfile(name)==nil)then
      KLS:Echo("-"..name.."- doesnt exist.")
      KLS:ListProfiles()
   else
      KLSdb:SetProfile('Default')
      KLSdb:DeleteProfile(name)
      KLS:Echo("-"..name.."- deleted. Now using Default profile..")
   end
end

function KLS:ListProfiles()
KLS:DebugMsg("ListProfiles()")

    print("KLS: Available Profiles: ");
    
	for i,k in ipairs(KLSdb:GetProfiles()) do
		if k == KLSdb:GetCurrentProfile() then
			print(' - ' .. k.." (current)")
		else
			print(' - ' .. k)
		end
	end
end
SLASH_KLS1 = "/kls"
SlashCmdList["KLS"] = function(msg)
   local cmd, arg = string.split(" ", msg, 2)
   cmd = string.lower(cmd or "")
   arg = arg or ""
   
   if (cmd == "version") then
      print("Keyboard Layout Saver v."..KLSversion)
   elseif (cmd=="list")then
      KLS:ListProfiles()
   elseif (cmd=="" or arg=="")then
      KLS:Echo("/kls delete <name>")
      KLS:Echo("/kls save <name>")
      KLS:Echo("/kls load <name>")
      KLS:Echo("/kls list")
      KLS:Echo("/kls version")
      KLS:Echo("/kls debug [on|off]")
   elseif (cmd=="save" and arg~="")then
      KLS:SaveProfil(arg)
   elseif (cmd=="debug" and arg~="")then   
      if arg == "on" then
        KLS:DebugWindowToggle(1)
      else
        KLS:DebugWindowToggle(0)
      end
   elseif (cmd=="load" and arg~="")then
      KLS:LoadProfil(arg)
   elseif (cmd=="delete" and arg~="")then
      KLS:DeleteProfil(arg)
   
   else    
      KLS:Echo("wrong entry")
   end
end

function KLS:DebugWindowToggle(visible)
   if visible == 1 then
      KLSdebugEnabled = 1
      KLSdb.char.KLSdebugEnabled=1
      
      KLSdebugframe = AceGUI:Create("Frame")
      KLSdebugframe:SetTitle("Keyboard Layout Saver Debug")
      KLSdebugframe:SetStatusText("Close to turn off full debug.")
      KLSdebugframe:SetCallback("OnClose", function(widget) AceGUI:Release(widget) KLSdebugEnabled = 0 KLSdb.char.KLSdebugEnabled=0 end)
      KLSdebugframe:SetLayout("Fill")
      KLSdebugframe:SetWidth("400")
      KLSdebugframe:SetHeight("400")
      KLS:DebugMsg("DebugWindowToggle(1)")
   else
   KLS:DebugMsg("DebugWindowToggle(0)")
      if KLSdebugEnabled == 1 then
          KLSdebugEnabled = 0
          KLSdb.char.KLSdebugEnabled=0
          KLSdebugframe:Release()
      end
   end
   
end

function KLS:DebugMsg(debugMessage)
   if KLSdebugbuffer == 0 then
      debugMessage = date("%H:%M:%S").." - DEBUGGER Loaded"
      table.insert(KLSdebugbuffer,debugMessage)
      
   else
      debugMessage = date("%H:%M:%S").." - "..debugMessage
      table.insert(KLSdebugbuffer,debugMessage)
   end
  
    if KLSdebugEnabled ~= 1 then 
        KLSdebugbuffer = circular_list(KLSdebugbuffer,50) -- Save 20 debug messages by default, even with debug turned off.
        return
    end
    
   KLSdebugframe:ReleaseChildren() -- Refresh textbox
   
   local editbox = AceGUI:Create("MultiLineEditBox")
   editbox:SetNumLines(10)
   local KLSdebugbuffervariable = "\n"
   for _,value in ipairs(KLSdebugbuffer) do 
        KLSdebugbuffervariable =  KLSdebugbuffervariable..value.."\n" 
   end
   editbox:SetText(KLSdebugbuffervariable)
   editbox:DisableButton(1)
   KLSdebugframe:AddChild(editbox)
end

-- Give this function a list_objec = {} and max amount of entries in that table
-- and it will remove them for you
function circular_list(list_object,list_max)

   if (#list_object > list_max)then
      while #list_object > list_max do 
         table.remove(list_object,1)
      end
   end
   
   return list_object
   
end

-- WoW Events are quite crazy, firing all over the place.
-- Give this function a the event and how many times per second
-- it is allowed to fire and it will return "IGNORE" if that event has gone haywire.
-- NOTE: global event_throttle_list will be needed for this function
-- NOTE 2: Should be called from the event itself
function event_throttle (event,localtime,seconds_threshold)

	-- Register the event
	--if 	run_counter[event] then
	--run_counter[times_run]++
	--run_counter[event] = { localtime = localtime, times_run = 0 }

end